// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FP_network_info_container_h
#define FP_network_info_container_h

/*
** Includes
*/
#include "FP_node_info_container.h"
#include <list>

/// Container class that will hold all information needed to build a data-flow network.
class FP_network_info_container
{
  /// Output operator
  friend ostream& operator<<(ostream& os, const FP_network_info_container& nic);


 private:
  /// The name of the network.
  string m_net_name;

  /// A flag that indicates if the network is an iterator network.
  bool m_iterator;

  /// A map object containing the node information structures (indexed by node names) for each node in the netowrk.
  map<string,FP_node_info_container> m_nodes_map;

  /// The name of the input node.
  string m_input_node_name;

  /// The name of the output node.
  string m_output_node_name;

  /// The name of the condition node. (for iterator nets only...)
  string m_condition_node_name;




 public:

  /*
  ** Constructors / Destructor
  */

  /** Constructor
   */
  FP_network_info_container();

  /** Constructor
   **@param net_name The name of the network to construct.
   */
  FP_network_info_container(const string& net_name, const bool iterator_net = false);


  /** Copy constructor
    */
  FP_network_info_container(const FP_network_info_container& netic);


  /** Desctructor
    */
  virtual ~FP_network_info_container();


  /** = operator
    */
  FP_network_info_container& operator=(const FP_network_info_container& nic);

  /*
  ** Accessors
  */

  /** Gets the name of the network.
   ** @return The name of the network as a string object.
    */
  string get_name() const;

  /** Sets the name of the network.
   ** @return void
    */
  void set_name(const string& s);
  
  /** Sets the input node of the network. (by name)
   ** @param input_node_name The name of the node to set as input node.
   ** @return void
   */
  void set_input_node_name(const string& input_node_name);


  /** Gets the name of the input node.
   ** @return The name of the input node, or an empty string if the input node is not set.
   */
  string get_input_node_name() const;


  /** Sets the output node of the network. (by name)
   ** @param output_node_name The name of the node to set as output node.
   ** @return void
   */
  void set_output_node_name(const string& output_node_name);


  /** Gets the name of the output node.
   ** @return The name of the output node, or an empty string if the output node is not set.
   */
  string get_output_node_name() const;

  /** Sets the condition node of the network. (by name)
   ** @param condition_node_name The name of the node to set as condition node.
   ** @return void
   */
  void set_condition_node_name(const string& condition_node_name);


  /** Gets the name of the condition node.
   ** @return The name of the condition node, or an empty string if the condition node is not set.
   */
  string get_condition_node_name() const;


  /** Adds a node information container to the network information container.
   ** @param node_name The name of the node.
   ** @return void
   ** @exception FileParserException
   */
  void add_node(const string& node_name) throw(FileParserException);


  /** Adds an input variable to a node.
   ** @param node_name The name of the node for which to add an input variable.
   ** @param ii A structure containing information about the input variable to add.
   ** @return void
   ** @exception FileParserException
   */
  void add_input_to_node(const string& node_name, const input_var_info& ii) throw(FileParserException);


  /** Gets the information about an input variable of a node.
   ** @param node_name The name of the node that contains the input variable.
   ** @input_name The name of the input variable.
   ** @return A structure containing the information about the input variable.
   ** @exception FileParserException
   */
  input_var_info get_input_from_node(const string& node_name, const string& input_name) const throw(FileParserException);


  /** Adds a parameter variable to a node.
   ** @param node_name The name of the node for which to add a parameter variable.
   ** @param pi A structure containing information about the parameter variable to add.
   ** @return void
   ** @exception FileParserException
   */
  void add_param_to_node(const string& node_name, const param_info& pi) throw(FileParserException);

  /** Modifies a parameter variable of a node.
   ** @param node_name The name of the node for which to modify a parameter variable.
   ** @param pi A structure containing the new information about the parameter.
   ** @return void
   ** @exception FileParserException
   */
  void modify_param_to_node(const string& node_name, const param_info& pi) throw(FileParserException);

  /**
   ** Gets the information about a parameter variable of a node.
   ** @param node_name The name of the node.
   ** @param param_name The parameter variable name.
   ** @return A structure containing the information about the parameter variable.
   ** @exception FileParserException
   */
  param_info get_param_from_node(const string& node_name, const string& param_name) const throw(FileParserException);


  /**
   ** Gets the information about parameter variables of a node.
   ** @param node_name The name of the node.
   ** @return A list of param_info structures each containing the information about the parameter variable.
   ** @exception FileParserException
   */
  list<param_info> get_params_from_node(const string& node_name) const throw(FileParserException);

  /** Gets the type of a node.
   ** @param node_name The name of the node.
   ** @return The type of node as a string object.
   ** @exception FileParserException
   */
  string get_type_of_node(const string& node_name) const throw(FileParserException);


  /** Sets the type of a node.
   ** @param node_name The name of the node to set the type of.
   ** @node_type The type that the node will be set to.
   ** @return void
   ** @exception FileParserException
   */
  void set_type_of_node(const string& node_name, const string& node_type) throw(FileParserException);


  /** Gets the names of the nodes in the network.
   ** @return A list of node names as string objects.
   */
  list<string> get_node_names() const;


  /** Gets the names of the input variables from a node in the network.
   ** @param node_name The name of the node.
   ** @return The names of the input variables of the node as string objects.
   ** @exception FileParserException
   */
  list<string> get_input_var_names_from_node(const string& node_name) const throw(FileParserException);


  /**
   ** Gets the names of the parameter variables from a node in the network.
   ** @param node_name The name of the node.
   ** @return The names of the parameter variables of the node as string objects.
   ** @exception FileParserException
   */
  list<string> get_param_var_names_from_node(const string& node_name) const throw(FileParserException);


  /** Deletes all parameters from a node.
   ** @param node_name The name of the node.
   ** @return void
   ** @exception FileParserException
   */
  void reset_params_from_node(const string& node_name) throw(FileParserException);

  /** Tests if the net is an iterator net.
   ** @return true if its an iterator net, else false.
   */
  bool is_iterator() const;
};

#endif
